package webapps.ajax1.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.sql.*;
import java.util.*;
import com.google.gson.*;
import java.io.InputStream;
import webapps.ajax1.entities.*;
import webapps.ajax1.model.*;
import java.text.*;

/**
 * Servlet que responde a las operaciones CRUD de jtable sobre entidad Test
 * @author grchere
 */
@WebServlet(name = "JsonTestServlet", urlPatterns = {"/listTest", "/insertTest", "/deleteTest", "/updateTest"})
public class JsonTestServlet extends HttpServlet {

    private static FBCon fb0 = FBCon.getFBCon(0);

    // para customizacion de gson
    // Configure a custom GSON
    private final static GsonBuilder gsonBuilder = new GsonBuilder();
    private static Gson cgson;
    
    static {  
        gsonBuilder.registerTypeAdapter(java.util.Date.class, new DateJsonAdapter()); // tratamiento de fechas acorde con jtable.org
        gsonBuilder.registerTypeAdapter(java.sql.Date.class, new DateJsonAdapter()); // tratamiento de fechas acorde con jtable.org
        cgson = gsonBuilder.create();
    }
    
    
    /**
     * Processes requests for both HTTP
     * <code>GET</code> and
     * <code>POST</code> methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.setContentType("application/json");
        int tipoOperacion = 0; // 0 nada, 1 insert, 2 delete, 3 update, 4 list
       
        PrintWriter out = response.getWriter();
        
        try {
            
            if ( request.getRequestURI().toLowerCase().endsWith("/listtest") ) { // list
                // hacer query, cargar lista, pasar a json, agregar returns y records
                tipoOperacion = 4; // 0 nada, 1 insert, 2 delete, 3 update, 4 list
//                List<Test> lt = new ArrayList<Test>();
                ResultSet rs = fb0.getResultSet("SELECT ID,DESCR as \"DESCRIPCION\",FECHA FROM TEST ORDER BY ID ASC");
//                JTableData<Test> jtd = new JTableData<Test>("OK",lt);
                //while(rs.next()) {
                //    lt.add(new Test(rs.getInt(1),rs.getString(2),rs.getDate(3)));
                //}
                //String json = cgson.toJson(jtd);
                String json = cgson.toJson(ResultSetJson.toJson(rs));
                fb0.close(rs);
                //TRACE
                System.out.println("JsonTestServlet list="+json+"=");
                out.write(json);
            } else if ( request.getRequestURI().toLowerCase().endsWith("/inserttest") ) { // insert
                // obtener string enviado, pasar fromjson, obtener obj Test, insertar en b.d. y devolver Result:OK + nuevo objeto insertado
                tipoOperacion = 1; // 0 nada, 1 insert, 2 delete, 3 update, 4 list
                //TRACE
                System.out.println("JsonTestServlet: fecha="+request.getParameter("fecha"));
                SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy",request.getLocale());
                SimpleDateFormat sdff = new SimpleDateFormat("dd-MMM-yyyy",Locale.US);
                try {
                    java.util.Date d = sdf.parse(request.getParameter("fecha"));
                    String sffecha = sdff.format(d);
                    Test newtest = new Test(new Integer(request.getParameter("id")),request.getParameter("descripcion"),d);
                    fb0.executeQuery("INSERT INTO TEST VALUES("+newtest.getId()+",'"+newtest.getDescripcion()+"','"+sffecha+"')");
                    JTableData<Test> jtd = new JTableData<Test>("OK",newtest);
                    String json = cgson.toJson(jtd);
                    //TRACE
                    System.out.println("JsonTestServlet insert="+json+"=");
                    //out.write(addResultAndRecords(json,"OK",1));
                    out.write(json);
                } catch(ParseException pe) {
                    JTableData<Test> jtd = new JTableData<Test>("ERROR","PARSE ERROR de ["+request.getParameter("fecha")+"] "+pe.getMessage());
                    String json = cgson.toJson(jtd);
                    //TRACE
                    System.out.println("JsonTestServlet PARSE ERROR="+json+"="+pe.getMessage());
                    out.write(json);
                }
            } else if ( request.getRequestURI().toLowerCase().endsWith("/deletetest") ) { // delete
                // se deberia pasar solo el id (clave primaria), hacer delete, devolver Result:OK
                tipoOperacion = 2; // 0 nada, 1 insert, 2 delete, 3 update, 4 list
                String sjon = request.getParameter("id");
                fb0.executeQuery("DELETE FROM TEST WHERE ID="+sjon);
                JTableData<Test> jtd = new JTableData<Test>("OK");
                String json = cgson.toJson(jtd);
                //TRACE
                System.out.println("JsonTestServlet delete="+json+"=");
                out.write(json);
            } else if ( request.getRequestURI().toLowerCase().endsWith("/updatetest") ) { // udpate
                // idem insert, pero haciendo update, devolver lo mismo
                tipoOperacion = 3; // 0 nada, 1 insert, 2 delete, 3 update, 4 list
                //TRACE
                System.out.println("JsonTestServlet:updatetest: fecha="+request.getParameter("fecha"));
                SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy",request.getLocale());
                SimpleDateFormat sdff = new SimpleDateFormat("dd-MMM-yyyy",Locale.US);
                try {
                    java.util.Date d = sdf.parse(request.getParameter("fecha"));
                    String sffecha = sdff.format(d);
                    Test newtest = new Test(new Integer(request.getParameter("id")),request.getParameter("descripcion"),d);
                    fb0.executeQuery("UPDATE TEST SET DESCR='"+newtest.getDescripcion()+"',FECHA='"+sffecha+"' WHERE ID = "+newtest.getId());
                    
                    JTableData<Test> jtd = new JTableData<Test>("OK");
                    String json = cgson.toJson(jtd);
                    //TRACE
                    System.out.println("JsonTestServlet update="+json+"=");
                    //out.write(addResultAndRecords(json,"OK",1));
                    out.write(json);
                } catch(ParseException pe) {
                    JTableData<Test> jtd = new JTableData<Test>("ERROR","PARSE ERROR de ["+request.getParameter("fecha")+"] "+pe.getMessage());
                    String json = cgson.toJson(jtd);
                    //TRACE
                    System.out.println("JsonTestServlet:updatetest: PARSE ERROR="+json+"="+pe.getMessage());
                    out.write(json);
                }
            } else {
                JTableData<Test> jtd = new JTableData<Test>("ERROR","URL no implementada!");
                String json = cgson.toJson(jtd);
                //TRACE
                System.out.println("JsonTestServlet uri??="+json+"=");
                out.write(json);
            }
        } catch(java.sql.SQLException e) {
            System.err.println("Error sql: "+e.getMessage());
            JTableData<Test> jtd = new JTableData<Test>("ERROR","SQL ERROR "+e.getMessage());
            String json = cgson.toJson(jtd);
            //TRACE
            System.out.println("JsonTestServlet SQL ERROR="+json+"=");
            out.write(json);
        } finally {
            out.close();
        }
        //TRACE
        System.out.println("JsonTestServlet FIN!! to="+tipoOperacion+" uri={"+request.getRequestURI()+"}");
    }
    
    
/******************************************************************    
 * METODOS UTILIZANDO ANTES DE IMPLEMENTAR CUSTOM JSON SERIALIZATION
 * utilizado para poder probar jtable hasta resolver correctamente la serializacion
 * 
    private String addResultAndRecords(final String json,final String result,int records) {
        StringBuilder sb = new StringBuilder();
        String plural = "";
        if ( records > 1 ) plural = "s";
        sb.append("{\"Result\":\"");
        sb.append(result);
        if ( records >= 1 ) {
            sb.append("\",\"Record");
            sb.append(plural);
            sb.append("\":");
            sb.append(json);
        }
        sb.append("}");
        return sb.toString();
    }
    private String addResultAndRecords(final String result) {
        StringBuilder sb = new StringBuilder();
        sb.append("{\"Result\":\"");
        sb.append(result);
        sb.append("}");
        return sb.toString();
    }
*************************************************************************/
    /*
     * No utilizar getParameter() antes de llamar a este metodo!
     * Si se obtiene toda la peticion enviada por el cliente, no puede 
     * usarse luego getParameter()
     */
    private String getStringFromRequest(HttpServletRequest request) {
        InputStream in;
        StringBuilder sb=new StringBuilder();
        int d;
        try {
            in=request.getInputStream();
            while((d=in.read()) != -1){
                sb.append((char)d);
            }
        }catch(IOException ioe) {
        }
        //TRACE
        System.out.println("JsonTestServlet getStringFromRequest="+sb.toString());
        return sb.toString();
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /**
     * Handles the HTTP
     * <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP
     * <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
