package webapps.ajax1.servlets;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.annotation.*;
import java.util.*;
import java.io.*;
import com.google.gson.*;
/**
 * Servlet que devuelve datos de session de usuario,
 * siempre y cuando exista una session de usuario abierta y activa
 * Atencion!: solo sirve para recuperar datos de tipo String (no recupera objetos u otros tipos)
 * INPUT:
 *     parametro "variables" con un conjunto de variables de session a solicitar
 *     separadas por "," (coma)
 * OUTPUT: Devuelve formato ajax:
 *   cuando todo Ok: { "Result":"OK","<variable session>":"<value>"}
 *   cuando hay error: { "Result":"ERROR","Message":"mensaje de error"}
 * @author G. Cherencio
 */
@WebServlet(name = "SessionDataServlet", urlPatterns = {"/session"})
public class SessionDataServlet extends HttpServlet {
    /**
     * Processes requests for both HTTP
     * <code>GET</code> and
     * <code>POST</code> methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.setContentType("application/json");
        String vars = request.getParameter("variables");
System.out.println("vars="+vars);        
        Map<String,String> mapv = new HashMap<String,String>();
        PrintWriter out = response.getWriter();
        if ( vars != null ) {
            HttpSession session = request.getSession(false);
            if ( session != null ) {
                for(String var : vars.split(",")) {
                    mapv.put(var,(String) session.getAttribute(var));
                }
                mapv.put("Result","OK");
            } else {
                mapv.put("Result","ERROR");
                mapv.put("Message","Usuario no autenticado!");
            }
        } else {
            mapv.put("Result","ERROR");
            mapv.put("Message","Atributo [variables] no existe!");
        }
        // convertir json, retornar
        Gson gson = new Gson();
        String json = gson.toJson(mapv);
        out.print(json);
System.out.println("json="+json+"=");        
        out.close();
    }

    
    /**
     * Handles the HTTP
     * <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP
     * <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
